

Global Editor:TEditor
Global NGui:NUI_Manager

Include "TEditorGuiMain.bmx"
Include "TEditorGuiObjects.bmx"

Type TEditor
	Field NuiGui:NUI_Manager
	Field GuiMain:TEditorGuiMain
	Field GuiObjects:TEditorGuiObjects
	
	Field CreationsDB:TEditorCreationsDB
	
	Field TreeImg:NUI_Image
	
	Field OneWindow% = 1
	
	Field Drawing% = 1
	
	Function InitCoreSystem()
		Editor = New TEditor
		Editor.PrepareGame()
		Editor.CreationsDB = TEditorCreationsDB.Create()
		Editor.InitGui()
	End Function
	
	Method InitGui()
		SetMaskColor(255,0,255)
		
		NuiGui = NUI_Manager.Create()
		NuiGui.SetTheme(NUI_Theme.LoadTheme("Media\Editor\Themes\WinXP\WinXP.ntc"))
		
		TreeImg = NUI_Image.Load("Media/Editor/menu_icon_1.png")
		
		NGui = NuiGui
		
		GuiMain = TEditorGuiMain.Create()
		GuiObjects = TEditorGuiObjects.Create()
	End Method
	
	Method PrepareGame()
		'------ disable menu and create test world
		Menu.StopMenu()
		WorldManager.LoadWorld("")
		WorldManager.ChangeWorld(0,0,0)
		Game.EnableGameUpdate = 1
		Game.EnableGameDraw = 1
		Game.GameRuning = 1
		WorldManager.FreePlayer()
		

		'---- reconfigure debug module
		Debug.EnableDebugInput = 0
		Debug.ShowBase = 1
		Debug.ShowFps = 1
		Debug.ShowResData = 1
		Debug.ShowLines = 1
		
		'----- disable in game cursor
		Game.EnableCursorDraw = 0
		Cursor.Free()
		Cursor = Null
		ShowMouse
		
	End Method
	
	Method FreeCoreSystem()
	
		GuiObjects.Free()
		GuiObjects = Null
		
		GuiMain.Free()
		GuiMain = Null
		
		NGui = Null
		
		NuiGui = Null 'wtf ?
	End Method
	
	Method Run%()
		Return Game.Run()
	End Method
	
	Method Update()
		If key.IsKeyDown(key_enter) Then
			Game.Update()
		End If

		'---------- hack :/
		'For Local eff:TEffect = EachIn Effects
		'	eff.Update()
		'Next
		'----------
	
		If key.IsKeyHit(key_escape) Then Game.StopEngine()
		
		If key.IsKeyHit(KEY_TAB) Then
			GuiMain.Window.SetVisible(Not GuiMain.Window._visible)
			If OneWindow = 0 Then
				GuiObjects.Window.SetVisible(Not GuiObjects.Window._visible)
			End If
			
			Debug.ShowBase = Not Debug.ShowBase
			Debug.ShowFps = Not Debug.ShowFps
			Debug.ShowResData = Not Debug.ShowResData
			Debug.ShowLines = Not Debug.ShowLines
			
			TileMap.DrawLines = Not TileMap.DrawLines
			
			Drawing = Not Drawing
		End If
		
		If Key.IsMouseDown(2) Then
			Camera.Move(-MouseXSpeed()/Camera.Zoom,-MouseYSpeed()/Camera.Zoom)
		End If
		
		'If GuiMain.MouseOnWindow() Or GuiObjects.MouseOnWindow() Then
		'	AnimateValue(NGui_alpha,NGui_alpha_max,0.05)
		'Else
		'	AnimateValue(NGui_alpha,NGui_alpha_min,0.05)
		'End If

		GuiMain.Update()
		GuiObjects.Update()
		
		NGui.Update()
	End Method
	
	Field NGui_alpha_min# = 0.4
	Field NGui_alpha_max# = 1
	Field NGui_alpha# = 1
	
	Method AnimateValue(in# Var,ch#,speed#)
		If Abs(ch - in) <= speed Then
			in = ch
			Return
		End If
		in = in + Sgn(ch - in) * speed * dt
	End Method
	
	Method Draw()
		Game.Draw(0)

		GuiObjects.Draw()

		SetBlend(ALPHABLEND)
		SetColor 0,0,0
		SetAlpha 0.2
		SetRotation 0
		SetScale 1,1
		Local mx% = Camera.GetWorldX(MouseX())
		Local my% = Camera.GetWorldY(MouseY())
		DrawText mx+" / "+my,MouseX()+20,MouseY()
		
		SetBlend(ALPHABLEND)
		SetColor 255,255,255
		SetAlpha NGui_alpha
		SetRotation 0
		SetScale 1,1
		
		NGui.Render()

		PollEvent()
	
		Flip 0
	End Method
	
End Type


Type TEditorCreationDB
	Field Group%
	Field TreeViewNode:NUI_TreeViewNode = Null
	Field VisibleName$
	Field CreationName$
	
	Field SnakeSize% = 3
	Field SingleType% = -1
	Field BonusType% = 0
	
	Field OtherType% = 0
	'1 - end game
	'2 - jaqua logo
	'3 - jgs logo
	
	Method InitTreeNode(Parent:NUI_Control)
		TreeViewNode = NUI_TreeViewNode.Create(VisibleName,NGui,Parent)
	End Method
	
	Function Create:TEditorCreationDB(Group%, OriginalName$,VisibleName$)
		Local cr:TEditorCreationDB = New TEditorCreationDB
		
		cr.Group = Group
		cr.VisibleName = VisibleName
		cr.CreationName = OriginalName
		
		Return cr
	End Function
	
	
End Type

Type TEditorCreationsDB

	Const GroupSmall% = 1
	Const GroupBig% = 2
	Const GroupPortals% = 3
	Const GroupBonus% = 4
	Const GroupOther% = 5
	Const GroupSnakes% = 6

	Field Objects:TList
	
	Field LastCreated:TEditorCreationDB
	
	Method CreateTreeViewList(Parent:NUI_Control,Group% = 0)
	
		For Local obj:TEditorCreationDB = EachIn Objects
			If ((Group > -1) And (Group = obj.Group)) Or (Group = -1) Then
				obj.InitTreeNode(Parent)
			End If
		Next
	
	End Method
	
	Method CreateObject:TCreation(Node:NUI_TreeViewNode,wx#,wy#)
	
		Local obj:TEditorCreationDB = Null
		
		For Local objs:TEditorCreationDB = EachIn Objects
			If objs.TreeViewNode = Node Then
				obj = objs
				Exit
			End If
		Next
		
		If obj = Null Then Return Null
		
		LastCreated = obj
		
		Select obj.Group
		Case GroupSmall,GroupBig,GroupBonus
			Local cr:TCreation = WorldManager.CreateFullCreationFromType(obj.CreationName,obj.SnakeSize,obj.SingleType,obj.BonusType)
			WorldManager.AddCreationToWorld(cr)
			cr.SetPosition(wx,wy)
			Return cr
			
		Case GroupSnakes
			Local cr:TCreation = TCreationSnakeGenerator.CreateSnake(obj.SnakeSize)
			WorldManager.AddCreationToWorld(cr)
			cr.SetPosition(wx,wy)
			Return cr

		Case GroupPortals
			Local typ% = 0
			If obj.CreationName = "Next" Then typ = 1
			If obj.CreationName = "Prev" Then typ = 2
			If obj.CreationName = "World" Then typ = 3
		
			Local cr:TCreationPortal = TCreationPortal(WorldManager.CreateFullCreationFromType("Portal"))
			WorldManager.AddCreationToWorld(cr)
			cr.SetPosition(wx,wy)
			cr.SetPortalType(typ)

		Case GroupOther
		
			Select obj.OtherType
			Case 1
				Local cr:TCreation = TCreationMenuSimple.Create(GetResImage("Menu_GameOver"),wx,wy)
				WorldManager.AddCreationToWorld(cr)
				Return cr
			Case 2
				Local cr:TCreation = TCreationMenuSimple.Create(GetResImage("jAquaLogo"),wx,wy)
				WorldManager.AddCreationToWorld(cr)
				Return cr
			Case 3
				Local cr:TCreation = TCreationMenuSimple.Create(GetResImage("JgsLogo"),wx,wy)
				WorldManager.AddCreationToWorld(cr)
				Return cr
			Case 4
				Local cr:TCreation = TCreationMenuSimple.Create(Null,wx,wy)
				WorldManager.AddCreationToWorld(cr)
				Return cr
			Case 5
				Local cr:TCreation = TCreationMenuSimple.Create(GetResImage("Menu_Presents"),wx,wy)
				WorldManager.AddCreationToWorld(cr)
				Return cr
			Default
			End Select
		
		Default
		End Select
		
	End Method
	
	Method Free()
		
	End Method
	
	Method AddToList(Group%,OrgName$,VisName$)
		Objects.AddLast(TEditorCreationDB.Create(Group,OrgName,VisName))
	End Method
	Method AddToList_Single(Group%,OrgName$,VisName$,SingleT%)
		Objects.AddLast(TEditorCreationDB.Create(Group,OrgName,VisName))
		TEditorCreationDB(Objects.Last()).SingleType = SingleT
	End Method
	Method AddToList_Bonus(Group%,OrgName$,VisName$,BonusT%)
		Objects.AddLast(TEditorCreationDB.Create(Group,OrgName,VisName))
		TEditorCreationDB(Objects.Last()).BonusType = BonusT
	End Method
	Method AddToList_Snake(Group%,OrgName$,VisName$,BonusT%)
		Objects.AddLast(TEditorCreationDB.Create(Group,OrgName,VisName))
		TEditorCreationDB(Objects.Last()).SnakeSize = BonusT
	End Method
	Method AddToList_Other(Group%,OrgName$,VisName$,OtherT%)
		Objects.AddLast(TEditorCreationDB.Create(Group,OrgName,VisName))
		TEditorCreationDB(Objects.Last()).OtherType = OtherT
	End Method
	
	Method InitList()
		AddToList(GroupSmall,"Virus","Virus")
		AddToList(GroupSmall,"Snail","Snail")
		AddToList_Single(GroupSmall,"Single","Single_1",1)
		AddToList_Single(GroupSmall,"Single","Single_2",2)
		AddToList_Single(GroupSmall,"Single","Single_3",3)
		AddToList_Single(GroupSmall,"Single","Single_4",4)
		AddToList_Single(GroupSmall,"Single","Single_5",5)
		AddToList(GroupSmall,"DNAVirus","DNAVirus")

		AddToList(GroupBig,"JellyFishSmall","JellyFishSmall")
		AddToList(GroupBig,"JellyFish","JellyFish")
		AddToList(GroupBig,"Star","Star")
		AddToList(GroupBig,"Shell","Shell")
		AddToList(GroupBig,"Snake","Snake")

		AddToList(GroupPortals,"Next","Portal To Next Level")
		AddToList(GroupPortals,"Prev","Portal From Prev Level")
		AddToList(GroupPortals,"World","Portal To Next World")

		AddToList_Bonus(GroupBonus,"Bonus","Shapes Bonus",1)
		AddToList_Bonus(GroupBonus,"Bonus","Life Bonus",2)
		AddToList_Bonus(GroupBonus,"Bonus","Life Double Bonus",3)
		AddToList_Bonus(GroupBonus,"Bonus","SpeedBonus",5)
		AddToList(GroupBonus,"SnakeAdd","SnakeAdd")

		AddToList_Other(GroupOther,"EndGame","End Game",1)
		AddToList_Other(GroupOther,"jAqua","jAqua Logo",2)
		AddToList_Other(GroupOther,"jgs","jgs Logo",3)
		AddToList_Other(GroupOther,"null","null",4)
		AddToList_Other(GroupOther,"Presents","Presents",5)

		AddToList_Snake(GroupSnakes,"Snake","Snake Mini 1",TCreationSnakeGenerator.SnakeSimple1)
		AddToList_Snake(GroupSnakes,"Snake","Snake Mini 2",TCreationSnakeGenerator.SnakeSimple2)
		AddToList_Snake(GroupSnakes,"Snake","Snake Mini 3",TCreationSnakeGenerator.SnakeSimple3)
		AddToList_Snake(GroupSnakes,"Snake","Snake Mini 4",TCreationSnakeGenerator.SnakeSimple4)
		AddToList_Snake(GroupSnakes,"Snake","Snake Mini 5",TCreationSnakeGenerator.SnakeSimple5)
		AddToList_Snake(GroupSnakes,"Snake","Snake Medium 1",TCreationSnakeGenerator.SnakeMedium1)
		AddToList_Snake(GroupSnakes,"Snake","Snake Medium 2",TCreationSnakeGenerator.SnakeMedium2)
		AddToList_Snake(GroupSnakes,"Snake","Snake Medium 3",TCreationSnakeGenerator.SnakeMedium3)
		AddToList_Snake(GroupSnakes,"Snake","Snake Medium 4",TCreationSnakeGenerator.SnakeMedium4)
		AddToList_Snake(GroupSnakes,"Snake","Snake Medium 5",TCreationSnakeGenerator.SnakeMedium5)
		AddToList_Snake(GroupSnakes,"Snake","Snake Hard 1",TCreationSnakeGenerator.SnakeHard1)
		AddToList_Snake(GroupSnakes,"Snake","Snake Hard 2",TCreationSnakeGenerator.SnakeHard2)
		AddToList_Snake(GroupSnakes,"Snake","Snake Hard 3",TCreationSnakeGenerator.SnakeHard3)
		AddToList_Snake(GroupSnakes,"Snake","Snake Hard 4",TCreationSnakeGenerator.SnakeHard4)
		AddToList_Snake(GroupSnakes,"Snake","Snake Hard 5",TCreationSnakeGenerator.SnakeHard5)
	End Method

	Function Create:TEditorCreationsDB()
		Local db:TEditorCreationsDB = New TEditorCreationsDB
		
		db.Objects = New TList
		db.InitList()
		
		Return db
	End Function
	
End Type

