

Type TAnimation

	'base
	Field id%
	Field animation_name$
	Field enable%

	'color
	Field as#,rs#,gs#,bs#,ae#,re#,ge#,be#,a#,r#,g#,b#,change_color_speed#

	'image
	Field image_res_name$
	Field Image:TResourceAnimImage
	
	'frames
	Field start_frame%
	Field end_frame%
	Field AnimationSpeed# ' in milisec per frame  , for example 1000/12 or 1000/25
	Field LastAnimationTime#
	Field Frame#
	
	'in testing, dont use :)
	'Field SmothAnimation% = 0

	Method Free()
		Image = Null
	End Method
	
	Method ColorAnimation()
		AnimateValue(a,ae,change_color_speed)
		AnimateValue(r,re,change_color_speed)
		AnimateValue(g,ge,change_color_speed)
		AnimateValue(b,be,change_color_speed)
		SetAlpha (a / 255.0)
		SetColor r,g,b
	End Method
	
	Method RestartColorAnimation()
		a = as
		r = rs
		g = gs
		b = bs
	End Method
	
	Method CalcFrame%(Frame#)
		If end_frame - start_frame = 0 Then Return 0
		Local fram% = Int(Frame Mod Float(end_frame - start_frame))
		Return start_frame + fram
	End Method
	
	Method Draw(x#,y#,scx# = 1.0,scy# = 1.0)
		If enable = 0 Then Return
		
		ColorAnimation()
		
		Local scxo# = Image.Scale_x
		Local scyo# = Image.Scale_y
			
		Image.Scale_x:*scx
		Image.Scale_y:*scy
	
		'If SmothAnimation = 1 Then
		'	Local fr# = Frame - Int(Frame)
		'	SetAlpha 1-fr
		'	Image.DrawAnimResource(x,y,1,CalcFrame(Frame))
		'	SetAlpha fr
		'	Image.DrawAnimResource(x,y,1,CalcFrame(Frame+1))
		'	SetAlpha 1
		'Else
			Image.DrawAnimResource(x,y,1,Frame)
		'End If
		
		Image.Scale_x = scxo
		Image.Scale_y = scyo

		If start_frame <> end_frame Then
			Local AnimDelta# = jMilliSecs() - LastAnimationTime
			If AnimDelta => AnimationSpeed  Then
				Frame:+ AnimDelta/AnimationSpeed
				LastAnimationTime = jMilliSecs()
				If Frame > end_frame Then Frame = start_frame
			End If
		End If
	End Method
End Type

Type TAnimationModel
	Field Animations:TList
	
	Method SetAnimation(name$)
		CancelSizeCachs()
		
		Local need_anim_group$

		For Local a:TAnimation = EachIn Animations
			If a.animation_name = name$ Then
				need_anim_group = a.image_res_name
				Exit
			End If
		Next
		
		For Local b:TAnimation = EachIn Animations
			If b.image_res_name = need_anim_group Then
				b.enable = 0
				If b.animation_name = name$ Then
					b.enable = 1
					b.RestartColorAnimation()
					b.Frame = b.start_frame
				End If
			End If
		Next
		
		Width()
		Height()
	End Method
	
	Field WidthCache# = -1
	Field HeightCache# = -1
	
	Method CancelSizeCachs()
		WidthCache = -1
		HeightCache = -1
	End Method
	
	Method Width#()
		If WidthCache = -1 Then
			Local width#
			For Local anim:TAnimation = EachIn Animations
				If anim.enable = 0 Then Continue
				width = anim.Image.Image().width
				If width > WidthCache Then WidthCache = width
			Next
		End If
		Return WidthCache
	End Method
	
	Method Height#()
		If HeightCache = -1 Then
			Local height#
			For Local anim:TAnimation = EachIn Animations
				If anim.enable = 0 Then Continue
				height = anim.Image.Image().height
				If height > HeightCache Then HeightCache = height
			Next
		End If
		Return HeightCache
	End Method
	
	Method GetAnimation:TAnimation(name$)
		For Local a:TAnimation = EachIn Animations
			If a.animation_name = name$ Then Return a
		Next
	End Method
	
	Method Free()
		For Local a:TAnimation = EachIn Animations
			a.Free
		Next
		Animations.Clear
		Animations = Null
	End Method
	
	Method Draw(x#,y#,scx#=1.0,scy#=1.0)
		For Local a:TAnimation = EachIn Animations
			'If Camera.BoxInViewPort(x,y,a.Image.Fr_x,a.Image.Fr_y) Then
				a.Draw(x,y,scx,scy)
			'End If
		Next
	End Method
	
	Function LoadAnimationModel:TAnimationModel(file_name$)
		Local img:TAnimationModel = New TAnimationModel
		
		Local file:TStream = ReadPakFile(file_name$)
		Assert file,"Cant load model file "+file_name
		
		Local header$ = ReadString(file,4)
		If header <> "j2dm" Then RuntimeError "model file header corrupt"
		
		Local format_version% = ReadInt(file)
		If format_version <> 1 Then RuntimeError "model format file doesnt support"
		
		Local anim_num% = ReadInt(file)
		
		img.Animations = New TList
		For Local i% = 0 To anim_num
		
			Local anim:TAnimation = New TAnimation
			
			anim.id = ReadInt(file)
			anim.animation_name = ReadLine(file)
			anim.start_frame = ReadInt(file)
			anim.end_frame = ReadInt(file)
			anim.image_res_name = ReadLine(file)
			anim.as = ReadInt(file)
			anim.rs = ReadInt(file)
			anim.gs = ReadInt(file)
			anim.bs = ReadInt(file)
			anim.ae = ReadInt(file)
			anim.re = ReadInt(file)
			anim.ge = ReadInt(file)
			anim.be = ReadInt(file)
			anim.change_color_speed = ReadFloat(file)
			anim.AnimationSpeed = 1000.0 / ReadFloat#(file)
			
			anim.Image = GetResAnimImage(anim.image_res_name)
			anim.LastAnimationTime = jMilliSecs()
			anim.Frame = anim.start_frame
			
			img.Animations.AddLast anim
			
		Next
		
		Local end_file$ = ReadString(file,7)
		If end_file <> "jformat" Then RuntimeError "End of file corrup"
		
		
		CloseFile(file)
		
		Return img
	End Function
	
End Type


