Type TCameraState
	Field x#,y#
	Field Zoom#
	Field GlobalAlpha#
	Field EnZoom%
	Field EnGlobalAlpha%
	Field EnScale%
	Field grx#,gry#
	Field scx#,scy#
	Field renderopt%
End Type

Type TCamera

	'camera coordinates
	Field camera_x#,camera_y#
	
	Field Zoom# = 1.0
	Field GlobalAlpha# = 1.0
	
	Field EnableZoom% = 1
	Field EnableGlobalAlpha% = 1
	Field EnableScale% = 0
	
	Field GrapX#
	Field GrapY#
	
	Field ScaleX# = 1.0
	Field ScaleY# = 1.0
	
	Field RenderOptimization% = 1
	
	Method GetState:TCameraState()
		Local s:TCameraState = New TCameraState
		s.x = camera_x
		s.y = camera_y
		s.Zoom = Zoom
		s.GlobalAlpha = GlobalAlpha
		s.EnZoom = EnableZoom
		s.EnGlobalAlpha = EnableGlobalAlpha
		s.EnScale = EnableScale
		s.grx = GrapX
		s.gry = GrapY
		s.scx = ScaleX
		s.scy = ScaleY
		Return s
	End Method
	
	Method SetState(s:TCameraState)
		'If s <> Null Then Return
		Assert s,"Camera Set State Fail"
		camera_x = s.x
		camera_y = s.y
		Zoom = s.Zoom
		GlobalAlpha = s.GlobalAlpha
		EnableZoom = s.EnZoom
		EnableGlobalAlpha = s.EnGlobalAlpha
		EnableScale = s.EnScale
		GrapX = s.grx
		GrapY = s.gry
		ScaleX = s.scx
		ScaleY = s.scy
	End Method
	
	Method Serealize()
		SerFloat(camera_x)
		SerFloat(camera_y)
		SerFloat(Zoom)
		SerFloat(GlobalAlpha)
		SerInt(EnableZoom)
		SerInt(EnableGlobalAlpha)
		SerInt(EnableScale)
		SerFloat(ScaleX)
		SerFloat(ScaleY)
	End Method
	
	'draw image with camera
	Method DrawImages(image:TImage,x#,y#,frame%=0)
		If image = Null Then
			RuntimeError "WTF ?! null image in camera.drawimages , please debug again :)"
		End If

		If RenderOptimization = 1 And Not ImageInViewPort(image,x,y) Then
			FpsCounter.CameraRenderOptimizationCount:+1
			Return
		End If
		
		If EnableGlobalAlpha = 1 Then SetAlpha(GetAlpha()*GetGlobalAlpha())
		
		If EnableZoom = 1 Then
			Local scx#,scy#
			GetScale scx,scy
			SetScale scx*GetZoom(),scy*GetZoom()
		End If
		
		If EnableScale = 1 And (EnableZoom = 0 Or GetZoom() = 1.0) Then
			Local scx#,scy#
			GetScale scx,scy
			SetScale scx*ScaleX,scy*ScaleY
		End If
		
		If EnableScale = 1 And EnableZoom = 1 Then
			RuntimeError "EnableScale and EnableZoom in camera ! , please fuck off this shit"
		End If
		
		DrawImage image,GetScrX(x),GetScrY(y),frame
	End Method
	
	Method GetZoom#()
		If EnableZoom = 1 Then
			Return Zoom
		Else
			Return 1.0
		End If
	End Method
	Method GetGlobalAlpha#()
		If EnableGlobalAlpha = 1 Then
			Return GlobalAlpha
		Else
			Return 1.0
		End If
	End Method
	
	'move camera
	Method Move(x#,y#)
		camera_x :+ x
		camera_y :+ y
	End Method
	
	'set camera positon
	Method SetXY(x#,y#)
		camera_x = x
		camera_y = y
	End Method
	
	'get camera position
	Method GetX#()
		Return camera_x
	End Method
	Method GetY#()
		Return camera_y
	End Method
	
	'get mouse position with camera
	Method GetMouseX#()
		Return GetWorldX(MouseX())
	End Method
	Method GetMouseY#()
		Return GetWorldY(MouseY())
	End Method
	
	'get screen coordinates from world
	Method GetScrX#(x#)
		Return (x - camera_x)*GetZoom()
	End Method
	Method GetScrY#(y#)
		Return (y - camera_y)*GetZoom()
	End Method

	'get world coordinates from screen
	Method GetWorldX#(x#)
		Return x/GetZoom() + camera_x
	End Method
	Method GetWorldY#(y#)
		Return y/GetZoom() + camera_y
	End Method
	
	'return 1 if cordinates in current viewport
	Method InViewPort%(x#,y#)
		Local x2#,y2#
		x2 = GetScrX(x)
		y2 = GetScrY(y)
		
		If x2 >= 0 And x2 <= GrapX And y2 >= 0 And y2 <= GrapY Then
			Return 1
		Else
			Return 0
		End If
	End Method
	Method BoxInViewPort%(x#,y#,Width#,Height#)
		Local x2#,y2#,x3#,y3#
		x2 = GetScrX(x + Width/2.0)
		y2 = GetScrY(y + Height/2.0)
		x3 = GetScrX(x - Width/2.0)
		y3 = GetScrY(y - Height/2.0)
		
		If x2 >= 0 And x3 <= GrapX And y2 >= 0 And y3 <= GrapY Then
			Return 1
		Else
			Return 0
		End If
	End Method
	Method BoxInViewPort2%(x3#,y3#,x2#,y2#)
		x2 = GetScrX(x2)
		x3 = GetScrX(x3)
		y2 = GetScrY(y2)
		y3 = GetScrY(y3)
		
		If x2 >= 0 And x3 <= GrapX And y2 >= 0 And y3 <= GrapY Then
			Return 1
		Else
			Return 0
		End If
	End Method
	Method ImageInViewPort%(Img:TImage,x#,y#)
	
		Local x2#,y2#,x3#,y3#
		x2 = GetScrX(x + Float(Img.width)/2.0)
		y2 = GetScrY(y + Float(Img.height)/2.0)
		x3 = GetScrX(x - Float(Img.width)/2.0)
		y3 = GetScrY(y - Float(Img.height)/2.0)
		
		If x2 >= 0 And x3 <= GrapX And y2 >= 0 And y3 <= GrapY Then
			Return 1
		Else
			Return 0
		End If
	End Method
	
	
	Function Create:TCamera()
		Local cam:TCamera = New TCamera
		cam.GrapX = GraphicsWidth()
		cam.GrapY = GraphicsHeight()
		
		Return cam
	End Function
	
End Type

'draw image (simply function ... maybe for simplify code :) )
Function DrawImg(image:TImage,x#,y#,frame%=0)
	Camera.DrawImages(image,x,y,frame)
End Function
