
Type TCollisionDouble
	Field Creation:TCreation
	Field CreationObject:TCreationObject
	
	Method Free()
		Creation = Null
		CreationObject = Null
	End Method
End Type

Type TCollisionRequest

	Field id%
	Field Filter_Eatable% = 1
	Field OnlyWithSnakeHead% = 0
	Field Filter_CreationGroups% = 1
	Field Creation:TCreation
	Field CreationObject:TCreationObject
	Field CollisionObjects:TList
	
	Method Free()
		For Local cro:TCollisionDouble = EachIn CollisionObjects
			cro.Free()
		Next
		CollisionObjects.Clear()
		CollisionObjects = Null
	End Method
	
	Method Add(Cr:TCreation,Obj:TCreationObject)
		Local cro:TCollisionDouble = New TCollisionDouble
		cro.Creation = cr
		cro.CreationObject = Obj
		CollisionObjects.AddLast(cro)
	End Method
	
	Function Create:TCollisionRequest()
		Local cor:TCollisionRequest = New TCollisionRequest
		cor.CollisionObjects = New TList
		Return cor
	End Function
	
End Type


Type TCollisionManager

	Field Requests:TList
	
	Field Lock%
	
	Method Free()
		If Lock = 1 Then UnLockCollisions()
		ClearCollisions()
		Requests = Null
	End Method
	
	Method Update()
		For Local cor:TCollisionRequest = EachIn Requests
			
			For Local cr:TCreation = EachIn Creations
				If cor.Creation = cr Then Continue
				If cor.Filter_CreationGroups = 1 And cr.CreationGroup <> 0 And cor.Creation.CreationGroups = cr.CreationGroups Then Continue
				
				If cr.CreationObjects = Null Then Continue
				
				If cor.OnlyWithSnakeHead = 1 Then
					If cr.CreationType = "Snake" Then
						Local cro:TCreationObject = cr.gco(0)

						Local distq# = (cro.x - cor.CreationObject.x) * (cro.x - cor.CreationObject.x) + (cro.y - cor.CreationObject.y)*(cro.y - cor.CreationObject.y)
						Local distq2# = (cor.CreationObject.spherecollision + cro.spherecollision) * (cor.CreationObject.spherecollision + cro.spherecollision)
					
						If distq < distq2 Then
							cor.Add(cr,cro)
						End If
					End If
				Else
					For Local cro:TCreationObject = EachIn cr.CreationObjects
						If cor.Filter_Eatable = 1 And cro.Eatable = 0 Then Continue

						Local distq# = (cro.x - cor.CreationObject.x) * (cro.x - cor.CreationObject.x) + (cro.y - cor.CreationObject.y)*(cro.y - cor.CreationObject.y)
						Local distq2# = (cor.CreationObject.spherecollision + cro.spherecollision) * (cor.CreationObject.spherecollision + cro.spherecollision)
					
						If distq < distq2 Then
							cor.Add(cr,cro)
						End If
					Next
				End If
				
			Next		
		Next
	End Method
	
	Method AddCollisionRequest%(Creation:TCreation,CreationObject:TCreationObject)
		If Lock = 1 Then RuntimeError "Add collision to locked manager"
		
		Local cr:TCollisionRequest = TCollisionRequest.Create()
		cr.CreationObject = CreationObject
		cr.Creation = Creation
		cr.id = Requests.Count()
		Requests.AddLast(cr)
		Return cr.id
	End Method
	
	Method GetCollisions:TCollisionRequest(id%)
		Return TCollisionRequest(Requests.ValueAtIndex(id))
	End Method

	Method LockCollisions()
		If Lock = 1 Then RuntimeError "Lock locked manager"
		Lock = 1
	End Method
	Method UnLockCollisions()
		If Lock = 0 Then RuntimeError "UnLock unlocked manager"
		Lock = 0
	End Method
	Method ClearCollisions()
		If Lock = 1 Then RuntimeError "Clear locked manager"
		For Local cor:TCollisionRequest = EachIn Requests
			cor.Free()
		Next
		Requests.Clear()
	End Method

	Function Create:TCollisionManager()
		Local cm:TCollisionManager = New TCollisionManager
		cm.Requests = New TList
		cm.Lock = 1
		Return cm
	End Function
End Type

