
Type TFpsCounter
	'global vars
	Field CurrentFpsReal% = 0
	Field CurrentFpsTick:Double = 0
	Field DeltaTimeReal:Double = 0
	
	Field GameUpdateTime:Double = 0
	Field GameDrawTime:Double = 0
	Field GameCollisionTime:Double = 0
	Field ResourceManagerUpdateTime:Double = 0

	'control game delta time var
	Field Controldt% = 1
	Field ControlByRealFps% = 0 'if 1 - use real fps, if 0 - use tick fps
	Field ControlDefaultFps:Double = 65.0

	'limit fps
	Field LimitFps% = 1
	Field LimitDefaultFps:Double = 60
	
	'private vars
	Field LastTime:Double = 0
	Field TickInSec:Double = 0
	Field SecTimerLastTime:Double = 0
	Field SecTimerDeltaTime:Double = 1000
	Field GameUpdateTimeLastTime:Double = 0
	Field GameDrawTimeLastTime:Double = 0
	Field GameCollisionTimeLastTime:Double = 0
	
	'delta timing
	Field LastDtSize% = 32
	Field LastDt#[LastDtSize + 1]
	
	'optimizations breaks
	Field CameraRenderOptimizationCount%
	Field CreationRenderOptimizationCount%
	Field CreationUpdateOptimizationCount%
	
	Method MorphDt#(value#)
		LastDt = LastDt[1..]
		LastDt = LastDt[..LastDtSize+1]
		LastDt[LastDtSize] = value

		Local a# = 0
		For Local i% = 0 To LastDtSize
			a:+LastDt[i]
		Next
		a = a / Float(LastDtSize+1)
		
		Return a
	End Method

	Method Free()
	End Method
	
	Method Update()
		Local Current_Time:Double = jMilliSecs()
		
		DeltaTimeReal = Current_Time - LastTime
		LastTime = Current_Time
		If DeltaTimeReal = 0 Then
			CurrentFpsTick = 1000
		Else
			CurrentFpsTick = 1000 / DeltaTimeReal
		End If
		
		TickInSec :+1
		If Current_Time - SecTimerLastTime > SecTimerDeltaTime Then
			CurrentFpsReal = TickInSec * ( 1000.0 / SecTimerDeltaTime )
			TickInSec = 0
			SecTimerLastTime = Current_Time
		End If
		
		If Controldt = 1 Then
			If ControlByRealFps = 1 Then
				If CurrentFpsReal > 1 Then
					dt = ControlDefaultFps / Double(CurrentFpsReal)
				Else
					dt = 1
				End If
			Else
				dt = ControlDefaultFps / Double(CurrentFpsTick)
			End If
			dt = MorphDt(dt)
		End If

		If LimitFps = 1 Then
			Local NormalDelta:Double = 1000.0 / LimitDefaultFps
			Local GameTime:Double = GameDrawTime + GameUpdateTime
			If GameTime < NormalDelta Then
				Local Time1:Double = jMilliSecs()
				While (NormalDelta - GameTime) > (jMilliSecs() - Time1)
				Wend
				dt = 1
			End If
		End If

		If dt < 0.01 Then dt = 1
		If dt > 10 Then dt = 10
	End Method
	
	Field UpdateCount%
	Field DrawCount%
	Field CollisionCount%
	
	Method UpdateStart()
		UpdateCount = 1
		GameUpdateTimeLastTime = jMilliSecs()
	End Method
	Method UpdateEnd()
		GameUpdateTime = jMilliSecs() - GameUpdateTimeLastTime
	End Method
	Method DrawStart()
		DrawCount = 1
		GameDrawTimeLastTime = jMilliSecs()
	End Method
	Method DrawEnd()
		GameDrawTime = jMilliSecs() - GameDrawTimeLastTime
	End Method
	Method CollisionStart()
		CollisionCount = 1
		GameCollisionTimeLastTime = jMilliSecs()
	End Method
	Method CollisionEnd()
		GameCollisionTime = jMilliSecs() - GameCollisionTimeLastTime
	End Method
	
	Method StartCountTime()
		UpdateCount = 0
		DrawCount = 0
		CollisionCount = 0
	End Method
	Method EndCountTime()
		If UpdateCount = 0 Then GameUpdateTime = 0
		If DrawCount = 0  Then GameDrawTime = 0
		If CollisionCount = 0 Then GameCollisionTime = 0
	End Method
	
	Field ProfTimes#[128]
	Field ProfCount%
	
	Method ProfilerStart()
		ProfCount = 1
		ProfTimes[0] = jMillisecs()
	End Method
	Method Profiler()
		ProfTimes[ProfCount-1] = jMillisecs() - ProfTimes[ProfCount-1]
		ProfTimes[ProfCount] = jMillisecs()
		ProfCount:+1
	End Method
	Method ProfilerEnd()
		ProfTimes[ProfCount-1] = jMillisecs() - ProfTimes[ProfCount-1]
	End Method
	
	Function CreateFpsCounter()
		FpsCounter = New TFpsCounter
		FpsCounter.LastTime = jMilliSecs()
		dt = 1
	End Function
End Type
