
Type TScoreLayerObject
	Field img:TResourceImage
	Field img_anim:TResourceAnimImage
	Field x#,y#
	Field need_x#,need_y#
	
	Method Free()
		img = Null
		img_anim = Null
	End Method
	
	Method Update(speed#)
		AnimateValue(x,need_x,speed)
		AnimateValue(y,need_y,speed)
	End Method
	
	Method Draw(scx#=1,scy#=1,crl% = 1)
		SetBlend ALPHABLEND
		If crl = 1 Then SetColor 255,255,255
		SetScale scx,scy
		SetRotation 0
		SetAlpha 1 * Game.PauseGameAlpha
		If img <> Null Then
			DrawImage img.Image(),x,y
		ElseIf img_anim <> Null Then
			DrawImage img_anim.Image(),x,y
		End If
	End Method
	
	Method SetXY(x2#,y2#,need% = 1)
		If need = 0 Then
			x = x2
			y = y2
		Else
			need_x = x2
			need_y = y2
		End If
	End Method
	
	Function Create:TScoreLayerObject(img:TResourceImage,img_anim:TResourceAnimImage = Null)
		Local obj:TScoreLayerObject = New TScoreLayerObject
		obj.img = img
		obj.img_anim = img_anim
		Return obj
	End Function
	
End Type

Type TScoreLayer
	Field Typ%
	Field Objects:TList
	
	Field Text_Score:TRasterText
	Field Text_Money:TRasterText
	Field MiniStar:TResourceImage
	
	Field LayerSpeed#
	Field LayerSpeedMax#
	Field LayerSpeedDefault#
	Field LayerSpeedTimeOff#
	Field LayerSpeedAnimSpeed# = 0.02
	
	Field BonusFirstAidCount% = 0
	Field BonusAntiDotCount% = 0
	
	Method Free()
		For Local obj:TScoreLayerObject = EachIn Objects
			obj.Free()
		Next
		Objects.Clear()
		Objects = Null
		
		If Text_Money <> Null Then Text_Money.Free()
		Text_Money = Null
		
		If Text_Score <> Null Then Text_Score.Free()
		Text_Score = Null
		
		MiniStar = Null
	End Method
	
	Method Update()
		If jMilliSecs() > LayerSpeedTimeOff Then
			AnimateValue(LayerSpeed,LayerSpeedDefault,LayerSpeedAnimSpeed)
		Else
			AnimateValue(LayerSpeed,LayerSpeedMax,LayerSpeedAnimSpeed)
		End If
		
		For Local obj:TScoreLayerObject = EachIn Objects
			obj.Update(LayerSpeed)
			If obj.x = obj.need_x And obj.y = obj.need_y Then
				GetXY(obj.need_x,obj.need_y)
			End If
		Next
		
		If Typ = 2 Then
			Text_Money.Update()
			Text_Score.Update()
		End If
		
		If Typ = 3 Then
			Local antidotcount% = 0
			Local firstaidcount% = 0

			'For Local Bonus:TBonus = EachIn Player.BonusManager.Bonuses
			'	If Bonus.BonusName = "AntiDotBonus" Then antidotcount:+1
				'If Bonus.BonusName = "FirstAidBonus" Then firstaidcount:+1
			'Next

			For Local Entry:TBonusListBoxEntry = EachIn Player.BonusShop.UserBonuses.Bonuses
				Local Bonus:TBonus = Entry.Bonus
				If Bonus.BonusName = "AntiDotBonus" Then antidotcount:+1
				If Bonus.BonusName = "FirstAidBonus" Then firstaidcount:+1
			Next
			
			If antidotcount <> BonusAntiDotCount Then
				If antidotcount > BonusAntiDotCount Then
					For Local i% = 1 To (antidotcount-BonusAntiDotCount)
						AddBonus(GetResAnimImage("Bonus_06_Image"))
					Next
				Else
					For Local i% = 1 To (BonusAntiDotCount-antidotcount)
						RemoveBonus(GetResAnimImage("Bonus_06_Image"))
					Next
				End If
				BonusAntiDotCount = antidotcount
			End If
			If firstaidcount <> BonusFirstAidCount Then
				If firstaidcount > BonusFirstAidCount Then
					For Local i% = 1 To (firstaidcount-BonusFirstAidCount)
						AddBonus(GetResAnimImage("Bonus_07_Image"))
					Next
				Else
					For Local i% = 1 To (BonusFirstAidCount-firstaidcount)
						RemoveBonus(GetResAnimImage("Bonus_07_Image"))
					Next
				End If
				BonusFirstAidCount = firstaidcount
			End If
			
		End If
	End Method
	
	Field PoisR# = 255
	Field PoisG# = 255
	Field PoisB# = 255
	
	Const ColorSp# = 2
	
	Method Draw()
		For Local obj:TScoreLayerObject = EachIn Objects
			If Typ=2 Then
				If Player And Player.Creation And Player.Creation.PoisionLevel > 0 Then
					AnimateValue(PoisR,255,ColorSp)
					AnimateValue(PoisG,128,ColorSp)
					AnimateValue(PoisB,128,ColorSp)
				Else
					AnimateValue(PoisR,255,ColorSp)
					AnimateValue(PoisG,255,ColorSp)
					AnimateValue(PoisB,255,ColorSp)
				End If
				
				SetColor PoisR,PoisG,PoisB
				obj.Draw(1.3,1.3,0)
				
				SetBlend ALPHABLEND
				SetAlpha 1 * Game.PauseGameAlpha
				SetColor 255,255,255
				SetScale 0.6,0.6
				SetRotation 0
				Text_Score.Draw(Camera.GetWorldX(obj.x),Camera.GetWorldY(obj.y))
				SetRotation 0
				SetColor 255,255,128
				Text_Money.Draw(Camera.GetWorldX(obj.x),Camera.GetWorldY(obj.y))
				
				'Local lvl% = Player.ScoreManager.SnakeLevel
				Local lvl% = Player.ScoreManager.Combo
				
				If lvl > 0 Then
					Local DeltaAngle# = -15
					Local Angle# = 90.0 - Float(lvl-1)*DeltaAngle/2.0
					Local Radius# = 32
					Local dx#,dy#
					
					SetBlend ALPHABLEND
					SetColor 255,255,255
					SetScale 0.6,0.6
					SetRotation 0
					
					For Local r% = 1 To lvl
						SetAlpha 1 * Game.PauseGameAlpha
						dx = Radius*Cos(Angle)
						dy = Radius*Sin(Angle)
						
						MiniStar.DrawResource(Camera.GetWorldX(obj.x) + dx,Camera.GetWorldY(obj.y) + dy)
						
						Angle:+DeltaAngle
					Next
					
					
				End If
			ElseIf Typ = 3 Then
				obj.Draw(0.3,0.3)
			Else
				obj.Draw()
			End If
		Next
	End Method
	
	Method func#(x#,x_size#)
		If x < x_size/1.5 Then 
			Return x-x/2
		Else
			Return x-(x_size-x)
		End If
	End Method
	
	Method AddBonus(image:TResourceAnimImage)
		Local obj:TScoreLayerObject = TScoreLayerObject.Create(Null,image)
		Objects.AddLast(obj)
		GetXY(obj.x,obj.y)
		obj.SetXY(obj.x,obj.y)
		
		'obj.need_x = Camera.GrapX - 100
		'obj.need_y = 100
		
		'obj.x = obj.need_x
		'obj.y = obj.need_y
		
	End Method
	Method RemoveBonus(image:TResourceAnimImage)
		For Local obj:TScoreLayerObject = EachIn Objects
			If obj.img_anim = image Then
				obj.Free()
				Objects.Remove(obj)
				obj = Null
				Return
			End If
		Next
	End Method
	
	Method GetXY(x# Var,y# Var)
		Local x0# = Camera.GrapX - 160 - 50 - 20
		Local y0# = Camera.GrapY - 160 - 50 - 20
		Local size# = 128 + 50
		If Typ = 3 Then
			x0 = TScoreLayerObject(Player.ScoreManager.ScoreLayer.Objects.First()).x - 32
			y0 = TScoreLayerObject(Player.ScoreManager.ScoreLayer.Objects.First()).y - 96
			size = 64
		
			x = Rand(x0,x0 + size)
			y = Rand(y0,y0 - size)
		Else
			x = Rand(x0,x0 + size)
			y = Rand(y0+size-func(x-x0,size),y0+size)
		End If
	End Method
	
	Field LastUpdatedTime#
	
	Function Create:TScoreLayer(obj_type%)
		Local layer:TScoreLayer = New TScoreLayer
		layer.Objects = New TList
		layer.Typ = obj_type
		
		Select obj_type
		Case 1
			For Local i% = 0 To 25
				Local obj:TScoreLayerObject = TScoreLayerObject.Create(GetResImage("UserScore_Ring"+Rand(1,4)))
				layer.Objects.AddLast(obj)
				layer.GetXY(obj.x,obj.y)
				obj.SetXY(obj.x,obj.y)
			Next
			layer.LayerSpeed = 0.3
			layer.LayerSpeedMax = 1
		Case 2
			Local obj:TScoreLayerObject = TScoreLayerObject.Create(GetResImage("UserScore_Ring5"))
			layer.Objects.AddLast(obj)
			layer.GetXY(obj.x,obj.y)
			obj.SetXY(obj.x,obj.y)
			layer.LayerSpeed = 0.2
			layer.LayerSpeedMax = 0.5
			
			layer.Text_Score = TRasterText.Create("0",GetResAnimImage("Font_1"),TRasterText.FontTypeNumbersCircle,14)
			layer.Text_Money = TRasterText.Create("0",GetResAnimImage("Font_1"),TRasterText.FontTypeNumbers,14)
			
			layer.MiniStar = GetResImage("UserScore_MiniStar")
		Case 3
			'Local obj:TScoreLayerObject = TScoreLayerObject.Create(GetResImage("UserScore_Ring"+Rand(1,4)))
			'layer.Objects.AddLast(obj)
			'layer.GetXY(obj.x,obj.y)
			'obj.SetXY(obj.x,obj.y)
			
			layer.LayerSpeed = 0.3
			layer.LayerSpeedMax = 1
		
		Default
		End Select
		
		
		layer.LayerSpeedDefault = layer.LayerSpeed
		
		Return layer
	End Function
End Type

Type TScoreManager

	Const AddScore% = 1
	Const AddMoney% = 2
	
	Field Score#
	Field Money#
	
	Field SnakeLevel%
	
	Field Combo%
	
	Field Back:TScoreLayer
	
	Field BonusLayer:TScoreLayer
	Field ScoreLayer:TScoreLayer
	
	'Field TextAppear:TTextAppear
	
	Method AddPoints(points%,typ% = TScoreManager.AddScore)
		Select typ
		Case TScoreManager.AddScore
			If Combo <> 0 Then points:*Combo
			Score :+ Float(points)
			ScoreLayer.LastUpdatedTime = jMilliSecs()
		Case TScoreManager.AddMoney
			Money :+ Float(points)
		Default
		End Select
		ScoreLayer.Text_Score.SetText(Int(Score))
		ScoreLayer.Text_Money.SetText(Int(Money))
		
		Back.LayerSpeedTimeOff = jMillisecs() + 500
		ScoreLayer.LayerSpeedTimeOff = jMillisecs() + 500
	End Method

	
	Method Update()
		If Player.Creation = Null Then Return
		
		ScoreLayer.Text_Score.SetText(Int(Score))
		ScoreLayer.Text_Money.SetText(Int(Money))
		
		Back.Update()
		BonusLayer.Update()
		ScoreLayer.Update()
		
		'TextAppear.Update()
		'If TextAppear.StateOn.StateName = "Stop" Then TextAppear.SetState("Start")

	End Method
	Method Draw()
		SetScale 1,1
		SetRotation 0
		SetAlpha 1 * Game.PauseGameAlpha
		SetBlend LIGHTBLEND
		SetColor 255,255,255
	
		'DrawText "Score : " + Int(Score),400,0
		'DrawText "Combo : " + Combo,400,20
		'DrawText "Money : " + Money,400,40
				
		Back.Draw()
		BonusLayer.Draw()
		ScoreLayer.Draw()
		
		'TextAppear.Draw(300,100)
	End Method
	
	Method Free()
		Back.Free()
		Back = Null
		
		ScoreLayer.Free()
		ScoreLayer = Null
		
		BonusLayer.Free()
		BonusLayer = Null
		
		'TextAppear.Free()
		'TextAppear = Null
	End Method
	
	Method Serealize()
		SerFloat(Score)
		SerInt(Combo)
		SerFloat(Money)
		SerInt(SnakeLevel)
		
		AddPoints(0)
	End Method
	
	Function Create:TScoreManager()
		Local sc:TScoreManager = New TScoreManager
		
		sc.Back = TScoreLayer.Create(1)
		sc.ScoreLayer = TScoreLayer.Create(2)
		sc.BonusLayer = TScoreLayer.Create(3)
		
		sc.Money = 0
		sc.Score = 0
		
		'sc.TextAppear = TTextAppear.Create(GetResImage("jAquaLogo"),0,0)
		
		Return sc
	End Function

	
End Type

