

Type TTextAppearState
	Field Text:TTextAppear
	Field StateName$
	
	Method Init(Generate% = 1) Abstract
	Method Free() Abstract
	Method Update() Abstract
	Method Draw() Abstract
	
	Method SaveGenerated(Dir$ = "") Abstract
End Type

Type TTextAppear
	Field x#,y#
	Field Img:TResourceImage
	Field StateOn:TTextAppearState
	Field States:TList
	
	Method SetState(Name$)
		For Local St:TTextAppearState = EachIn States
			If St.StateName = Name Then
				StateOn = St
				St.Init()
			End If
		Next
	End Method

	Method Free()
		StateOn = Null
		For Local St:TTextAppearState = EachIn States
			St.Free()
		Next
		States.Clear()
		States = Null
		Img = Null
	End Method
	
	Method Update()
		If StateOn <> Null Then StateOn.Update()
	End Method
	
	Method Draw(x2#,y2#)
		x = x2
		y = y2
		If StateOn <> Null Then StateOn.Draw()
	End Method
	
	Method GenerateStates(Vertical%,Generate%)
		States.AddLast(TTextAppearStartState.Create(Self,Vertical))
		States.AddLast(TTextAppearStartMain.Create(Self))
		States.AddLast(TTextAppearStartEnd.Create(Self,Vertical))
		States.AddLast(TTextAppearStartStop.Create(Self))
		
		Img.Image()
		For Local St:TTextAppearState = EachIn States
			St.Init(Generate)
		Next
	End Method
	
	Method SaveGenerated(Dir$)
		For Local St:TTextAppearState = EachIn States
			St.SaveGenerated(Dir)
		Next
	End Method
	
	Function Create:TTextAppear(Img:TResourceImage,x#,y#,Generate% = 1,Vertical% = 1,base_state$ = "Start")
		Local t:TTextAppear = New TTextAppear
		t.States = New TList
		t.Img = Img
		t.x = x
		t.y = y
		t.GenerateStates(Vertical,Generate)
		t.SetState(base_state)
		Return t
	End Function
End Type

Type TTextAppearStartState Extends TTextAppearState
	Field NextState$
	Field Vertical%
	
	Field State%
	Field StateStart%
	Field NullDelta#
	Field NullDelta2#
	
	Field Timer#
	Field DeltaTime#
	
	Field BlurImages:TList

	Method Init(Generate% = 1)
		If BlurImages = Null Then
			BlurImages = New TList
			
			If Generate = 1 Then
				If Vertical = 1 Then
					BlurImages.AddLast(VerticalBlur(Text.Img.Image(),80))
					BlurImages.AddLast(VerticalBlur(Text.Img.Image(),70))
					BlurImages.AddLast(VerticalBlur(Text.Img.Image(),60))
				Else
					BlurImages.AddLast(HorizontalBlur(Text.Img.Image(),80))
					BlurImages.AddLast(HorizontalBlur(Text.Img.Image(),70))
					BlurImages.AddLast(HorizontalBlur(Text.Img.Image(),60))
				End If
			Else
				If Vertical = 1 Then
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurV80").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurV70").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurV60").Image())
				Else
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurH80").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurH70").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurH60").Image())
				End If
			End If
		End If
		
		State = StateStart
		Timer = -1
	End Method
	
	Method SaveGenerated(Dir$ = "")
		Local file:TStream = WriteFile(Dir+"Resources.txt")
		Local v$ = "H"
		If Vertical = 1 Then v = "V"
		file.WriteLine("NewImageRes("+Chr(34)+Dir+Text.Img.Name+"_Blur"+v+"80.png"+Chr(34)+","+Chr(34)+Text.Img.Name+"_Blur"+v+"80"+Chr(34)+")")
		file.WriteLine("NewImageRes("+Chr(34)+Dir+Text.Img.Name+"_Blur"+v+"70.png"+Chr(34)+","+Chr(34)+Text.Img.Name+"_Blur"+v+"70"+Chr(34)+")")
		file.WriteLine("NewImageRes("+Chr(34)+Dir+Text.Img.Name+"_Blur"+v+"60.png"+Chr(34)+","+Chr(34)+Text.Img.Name+"_Blur"+v+"60"+Chr(34)+")")
		file.Close()
		file = Null
		
		SavePixmapPNG(TImage(BlurImages.ValueAtIndex(0)).Lock(0,1,0),Dir+Text.Img.Name+"_Blur"+v+"80.png")
		SavePixmapPNG(TImage(BlurImages.ValueAtIndex(1)).Lock(0,1,0),Dir+Text.Img.Name+"_Blur"+v+"70.png")
		SavePixmapPNG(TImage(BlurImages.ValueAtIndex(2)).Lock(0,1,0),Dir+Text.Img.Name+"_Blur"+v+"60.png")
	End Method
	
	Method Free()
		Text = Null
		
		If BlurImages <> Null Then
			BlurImages.Clear()
			BlurImages = Null
		End If
	End Method
	
	Method Update()
		If Timer = -1 Then Timer = jMillisecs()
	
		If (jMilliSecs() - Timer) > DeltaTime Then
			Timer = jMillisecs()
			State:-1
		End If
	
		If State < 1 Then
			Text.SetState(NextState)
			Timer = -1
		End If
	End Method
	
	Method Draw()
		Local delta# = (Float(State)/Float(StateStart))
		Local delta2# = delta*(NullDelta/2.0)
		Local delta3# = (Abs(Sgn(Float(State)/2.0-1))*2-1)*delta
		
		SetBlend ALPHABLEND
		SetAlpha (Float(StateStart-State+1)/Float(StateStart))*0.8
		SetColor 255,255,255
		SetScale 1,1
		SetRotation 0
		
		DrawImg TImage(BlurImages.ValueAtIndex(StateStart-State)),Text.x+(NullDelta2*delta3-delta2)*(Abs(Vertical-1)),Text.y+(NullDelta2*delta3-delta2)*Vertical
		DrawImg TImage(BlurImages.ValueAtIndex(StateStart-State)),Text.x+(NullDelta2*delta3-delta2)*(Abs(Vertical-1)),Text.y+(NullDelta2*delta3+delta2)*Vertical
	End Method
	
	Function Create:TTextAppearStartState(Text:TTextAppear,Vertical% = 1,NextState$ = "Main")
		Local st:TTextAppearStartState = New TTextAppearStartState
		st.Text = Text
		st.StateName = "Start"
		
		st.NextState = NextState
		st.Vertical = Vertical
		st.StateStart = 3
		If Vertical = 1 Then
			st.NullDelta = 16
		Else
			st.NullDelta = 32
		End If
		st.NullDelta2 = 32
		
		st.DeltaTime = 45
		
		Return st
	End Function
End Type


Type TTextAppearStartMain Extends TTextAppearState
	Field BaseTime#
	Field TimeStart#
	Field NextState$
	Method Init(Generate% = 1)
		TimeStart = -1
	End Method
	Method Free()
		Text = Null
	End Method
	Method SaveGenerated(Dir$ = "")
	End Method
	Method Update()
		If TimeStart = -1 Then TimeStart = jMillisecs()
		
		If (jMillisecs() - TimeStart) > BaseTime Then
			TimeStart = -1
			Text.SetState(NextState)
		End If
	End Method
	Method Draw()
		SetBlend ALPHABLEND
		SetAlpha 1
		SetColor 255,255,255
		SetScale 1,1
		SetRotation 0
		Text.Img.DrawResource(Text.x,Text.y)
	End Method
	
	Function Create:TTextAppearStartMain(Text:TTextAppear,BaseTime# = 600,NextState$ = "End")
		Local st:TTextAppearStartMain = New TTextAppearStartMain
		st.Text = Text
		st.StateName = "Main"
		st.BaseTime = BaseTime
		st.NextState = NextState
		Return st
	End Function
End Type

Type TTextAppearStartEnd Extends TTextAppearState
	Field NextState$
	Field Vertical%
	
	Field State%
	Field StateStart%
	Field NullDelta#
	Field NullDelta2#
	
	Field Timer#
	Field DeltaTime#
	
	Field BlurImages:TList

	Method Init(Generate% = 1)
		If BlurImages = Null Then
			BlurImages = New TList
			
			If Generate = 1 Then
				If Vertical = 1 Then
					BlurImages.AddLast(VerticalBlur(Text.Img.Image(),80))
					BlurImages.AddLast(VerticalBlur(Text.Img.Image(),70))
					BlurImages.AddLast(VerticalBlur(Text.Img.Image(),60))
				Else
					BlurImages.AddLast(HorizontalBlur(Text.Img.Image(),80))
					BlurImages.AddLast(HorizontalBlur(Text.Img.Image(),70))
					BlurImages.AddLast(HorizontalBlur(Text.Img.Image(),60))
				End If
			Else
				If Vertical = 1 Then
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurV80").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurV70").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurV60").Image())
				Else
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurH80").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurH70").Image())
					BlurImages.AddLast(GetResImage(Text.Img.Name+"_BlurH60").Image())
				End If
			End If
		End If
		
		State = StateStart
		Timer = -1
	End Method
	Method SaveGenerated(Dir$ = "")
	End Method
	Method Free()
		Text = Null
		
		If BlurImages <> Null Then
			BlurImages.Clear()
			BlurImages = Null
		End If
	End Method
	
	Method Update()
		If Timer = -1 Then Timer = jMillisecs()
	
		If (jMilliSecs() - Timer) > DeltaTime Then
			Timer = jMillisecs()
			State:-1
		End If
	
		If State < 1 Then
			Text.SetState(NextState)
			Timer = -1
		End If
	End Method
	
	Method Draw()
		Local delta# = (Float(StateStart-State)/Float(StateStart))
		Local delta2# = delta*(NullDelta/2.0)
		Local delta3# = (Abs(Sgn(Float(State)/2.0-1))*2-1)*delta
		
		SetBlend ALPHABLEND
		SetAlpha (Float(State+1)/Float(StateStart))*0.8
		SetColor 255,255,255
		SetScale 1,1
		SetRotation 0
		
		DrawImg TImage(BlurImages.ValueAtIndex(StateStart-State)),Text.x+(NullDelta2*delta3-delta2)*(Abs(Vertical-1)),Text.y+(NullDelta2*delta3-delta2)*Vertical
		DrawImg TImage(BlurImages.ValueAtIndex(StateStart-State)),Text.x+(NullDelta2*delta3-delta2)*(Abs(Vertical-1)),Text.y+(NullDelta2*delta3+delta2)*Vertical
		
	End Method
	
	Function Create:TTextAppearStartEnd(Text:TTextAppear,Vertical% = 1,NextState$ = "Stop")
		Local st:TTextAppearStartEnd = New TTextAppearStartEnd
		st.Text = Text
		st.StateName = "End"
		
		st.NextState = NextState
		st.Vertical = Vertical
		st.StateStart = 3
		If Vertical = 1 Then
			st.NullDelta = 24
		Else
			st.NullDelta = 48
		End If
		st.NullDelta2 = 64
		
		st.DeltaTime = 45
		
		Return st
	End Function
End Type

Type TTextAppearStartStop Extends TTextAppearState
	Method Init(Generate% = 1)
	End Method
	Method Free()
		Text = Null
	End Method
	Method Update()
	End Method
	Method Draw()
	End Method
	Method SaveGenerated(Dir$ = "")
	End Method
	
	Function Create:TTextAppearStartStop(Text:TTextAppear)
		Local st:TTextAppearStartStop = New TTextAppearStartStop
		st.Text = Text
		st.StateName = "Stop"
		
		Return st
	End Function
End Type

