Type TSystemResource
	Field Client_Version% = -1
	Field Server_Version% = -1
	Field ResourceType$ = 0
	Field FlagToRemove% = 0
	Field ResourceFile$
	Field Upd:TSystemUpdate
	
	Method Free()
		Upd = Null
	End Method
	
	Method Upgrade() Abstract
	Method GetSize%() Abstract
	Method CanUpdate%() Abstract

End Type

Type TSystemResourceFile Extends TSystemResource
	Field Size% = -1

	Method Upgrade()
		If Client_Version = Server_Version Then Return

		If Server_Version = -1 Then
			DeleteFile Upd.ClientDir + ResourceFile
			FlagToRemove = 1
			Client_Version = Server_Version
		Else
			Upd.GetServerFile(ResourceFile,Upd.ClientDir + ResourceFile)
			Client_Version = Server_Version
		End If
	End Method
	
	Method GetSize%()
		If Size = -1 Then Size = Upd.GetServerFileSize(ResourceFile)
		Return Size
	End Method
	
	Method CanUpdate%()
		If Client_Version <> Server_Version Then Return 1
	End Method
	
	Function Create:TSystemResource(Upd:TSystemUpdate)
		Local res:TSystemResource = New TSystemResourceFile
		res.ResourceType = "f"
		res.Upd = Upd
		Return res
	End Function
End Type

Type TSystemResourceDir Extends TSystemResource
	Method Upgrade()
		If Client_Version = Server_Version Then Return
		
		If Server_Version = -1 Then
			DeleteDir Upd.ClientDir + ResourceFile,True
			FlagToRemove = 1
			Client_Version = Server_Version
		Else
			CreateDir(Upd.ClientDir + ResourceFile,1)
			Client_Version = Server_Version
		End If
	End Method
	
	Method GetSize%()
		Return 0
	End Method

	Method CanUpdate%()
		If Client_Version <> Server_Version Then Return 1
	End Method
	
	Function Create:TSystemResource(Upd:TSystemUpdate)
		Local res:TSystemResource = New TSystemResourceDir
		res.ResourceType = "d"
		res.Upd = Upd
		Return res
	End Function
End Type

Type TSystemUpdateInfo
	Field CanUpdate%
	Field UpdateSize%
	Field UpdatedFiles:TList
	Field TotalFiles%
	Field UpdateGlobal# 'from 1 to 100
	Field UpdateCurrentFile# 'from 1 to 100
	
	Field UpdateGurrentFileLast#
	Field DownloadedTotal#
	Field DownloadedCurrent#
	
	Method Update()
		'prbProgressBar1.SetValue(UpdateGlobal / 100)
		If UpdateGurrentFileLast = UpdateCurrentFile Then Return
		
		UpdateCurrentFile = UpdateCurrentFile
		
		DrawUpdateScreen(UpdateGlobal)
	End Method
	
	Method CalculateGlobal()
		UpdateGlobal = (Float(UpdatedFiles.Count()) / Float(TotalFiles)) - (1 - (UpdateCurrentFile / 100)) / Float(TotalFiles)
		UpdateGlobal :* 100
	End Method
	
	Method AddFile(name$,size%)
		UpdatedFiles.AddLast name
	End Method
	
	Method Free()
		UpdatedFiles.Clear()
		UpdatedFiles = Null
	End Method
	
	Function Create:TSystemUpdateInfo()
		Local ins:TSystemUpdateInfo = New TSystemUpdateInfo
		ins.UpdatedFiles = New TList
		Return ins
	End Function
End Type

Type TSystemUpdate
	Field UpdateServer$ = "http://jaqua/jAqua/scripts/game.php"
	Field SystemFile$ = "system.jupd"
	Field ClientDir$ = ""
	Field Resources:TList
	
	Field UpdateKey$
	
	Field Info:TSystemUpdateInfo

	Method UpdateClientDB%()
		Local cf:TStream = WriteFile(ClientDir + SystemFile)
		
		If cf = Null Then Return -1
		
		For Local r:TSystemResource = EachIn Resources
			Local st1:String = String.FromInt(r.Client_Version)
			Local l% = 4 - Len(st1)
			For Local i% = 1 To l
				st1 = "0"+st1
			Next
			cf.WriteLine(r.ResourceType+st1+";"+r.ResourceFile)
		Next
		
		cf.Close()
		cf = Null
		Return 1
	End Method
	
	Method GetServerRes%()
		Local sf:TStream = GetServerFile(SystemFile)

		If sf = Null Then Return -1
		
		While Not Eof(sf)
			Local line:String = sf.ReadLine()
			Local typ$ = Mid(line,1,1)
			Local version% = Mid(line,2,4).ToInt()
			Local file_name$ = Mid(line,7)
			
			Local Res:TSystemResource
			
			Select typ
			Case "f"
				Res = TSystemResourceFile.Create(Self)
			Case "d"
				Res = TSystemResourceDir.Create(Self)
			Default
			End Select
			
			Res.Server_Version = version
			Res.ResourceFile = file_name
			
			Resources.AddLast Res
		Wend
		
		sf = Null
		Return 1
	End Method
	
	Method GetClientRes%()
		Local cf:TStream = ReadFile(ClientDir + SystemFile)
		
		If cf = Null Then Return 1
		
		While Not Eof(cf)
			Local line:String = cf.ReadLine()
			Local typ$ = Mid(line,1,1)
			Local version% = Mid(line,2,4).ToInt()
			Local file_name$ = Mid(line,7)
			
			Local make% = 0
			
			For Local r:TSystemResource = EachIn Resources
				If r.ResourceFile = file_name Then
					r.Client_Version = version
					make = 1
					Exit
				End If
			Next
			
			If make = 0 Then
				Local Res:TSystemResource
			
				Select typ
				Case "f"
					Res = TSystemResourceFile.Create(Self)
				Case "d"
					Res = TSystemResourceDir.Create(Self)
				Default
				End Select
			
				Res.Client_Version = version
				Res.ResourceFile = file_name
				
				Resources.AddLast Res
			End If
			
		Wend
		
		cf = Null
		Return 1
	End Method
	
	Method UpgradeAllRes()
		For Local r:TSystemResource = EachIn Resources
			If r.CanUpdate() And r.ResourceType = "f" And r.Server_Version > -1 Then Info.AddFile(r.ResourceFile,r.GetSize())
			r.Upgrade()
			If r.FlagToRemove = 1 Then Resources.Remove(r)
			UpdateClientDB()
		Next
	End Method
	
	Method GetServerFile:TStream(file_name$,file_write_name$ = "")
		Local url$ = UpdateServer
		
		url$ = url$.Replace("http://", "")
		Local slashPos% = url$.Find("/"), host$, file$
	
		If slashPos <> -1
			host$ = url$[..slashPos]
			file$ = url$[slashPos..]
		Else
			Return Null
		EndIf
	
		Local fileStream:TStream
		
		If file_write_name = "" Then
			Local bank:TBank = CreateBank()
			fileStream:TStream = CreateBankStream(bank)
		Else
			fileStream:TStream = WriteFile(file_write_name)
		End If
		
		If Not(fileStream) Then
			Return Null
		End If
	
		Local TCPStream:TStream = OpenStream("tcp::" + host$)
		If Not(TCPStream)
			fileStream.Close()
			Return Null
		EndIf
	
		Local combined$ = "com=update&key="+UpdateKey+"&file="+file_name$
		
		WriteLine(TCPStream,"POST "+file$+" HTTP/1.0")
		WriteLine(TCPStream,"Accept: */*")
		WriteLine(TCPStream,"Host: "+host$)
		WriteLine(TCPStream,"Content-type: application/x-www-form-urlencoded")
		WriteLine(TCPStream,"User-agent: jaqua") 
		WriteLine(TCPStream,"Content-length: " + combined.length)
		WriteLine(TCPStream,"Pragma: no-cache")
		WriteLine(TCPStream,"Connection: close")
		WriteLine(TCPStream,"")
		WriteLine(TCPStream,combined)
		WriteLine(TCPStream,"")
	
		Delay 5
		Local dataLength% = -2
		Local in$ = TCPStream.ReadLine()
	
		If in$.Find("200") < 0
			TCPStream.Close()
			fileStream.Close()
			RuntimeError "Unknown error with file " + file_name
			Return Null
		EndIf
		
		While in$
			'DebugLog in$
			
			If in$.Find("Content-Length:") <> -1
				dataLength = Int(in$[in$.Find(":") + 1..].Trim())
			EndIf

			in$ = TCPStream.ReadLine()
		Wend
		
		If dataLength = -2 Then RuntimeError "wrong packet"
		
		If dataLength = -1 Then 'something like error
			While Not(TCPStream.Eof())
				If in$.Find("NoFile") <> -1
					RuntimeError "File : "+file_name$+" doesnt found"
				End If

				If in$.Find("NoUpdate") <> -1
					RuntimeError "Updates dosnt aviable now, please try later"
				End If

				If in$.Find("Fail") <> -1
					RuntimeError "Update Fail (get file)"
				End If
				
				in$ = TCPStream.ReadLine()
			Wend
			RuntimeError "Unknown error on file "+file_name$
		End If
		
		Delay 5
	
		Local byteCount%
		Local t% = 0
		While Not(TCPStream.Eof())
			fileStream.WriteByte(TCPStream.ReadByte())
		
			If t > 4096 Then
				t = 0
				Info.UpdateCurrentFile = ((byteCount / Float(dataLength - 1)) * 100)
				Info.CalculateGlobal()
				Info.Update()
			End If
			t:+1
		
			byteCount :+ 1
			
			Info.DownloadedCurrent = byteCount
		Wend
		Info.DownloadedTotal :+ Info.DownloadedCurrent
		Info.DownloadedCurrent = 0
		
	
		TCPStream.Close()
		If file_write_name = "" Then
			fileStream.Seek(0)
		Else
			fileStream.Close()
		End If
	
		If byteCount <> dataLength Then
			Return Null
		End If

		Return fileStream
	End Method

	Method GetServerFileSize%(file_name$)
		Local url$ = UpdateServer

		url$ = url$.Replace("http://", "")
		Local slashPos% = url$.Find("/"), host$, file$
	
		If slashPos <> -1
			host$ = url$[..slashPos]
			file$ = url$[slashPos..]
		Else
			Return -1
		EndIf
	
		Local stream:TStream = OpenStream("tcp::" + host$)
		If Not stream Then Return -1
	
		Local combined$ = "com=update&key="+UpdateKey+"&file="+file_name$
		
		WriteLine(stream,"POST "+file$+" HTTP/1.0")
		WriteLine(stream,"Accept: */*")
		WriteLine(stream,"Host: "+host$)
		WriteLine(stream,"Content-type: application/x-www-form-urlencoded")
		WriteLine(stream,"User-agent: jaqua") 
		WriteLine(stream,"Content-length: " + combined.length)
		WriteLine(stream,"Pragma: no-cache")
		WriteLine(stream,"Connection: close")
		WriteLine(stream,"")
		WriteLine(stream,combined)
		WriteLine(stream,"")

	
		While Not Eof(stream)
			Local in$ = stream.ReadLine()

			If in$.Find("Content-Length:") <> -1
				stream.Close()
				Return Int(in$[in$.Find(":") + 1..].Trim())
			EndIf	
			
			If in$.Find("NoFile") <> -1
				RuntimeError "File : "+file_name$+" doesnt found"
			End If

			If in$.Find("NoUpdate") <> -1
				RuntimeError "Updates dosnt aviable now, please try later"
			End If

			If in$.Find("Fail") <> -1
				RuntimeError "Update Fail (get file)"
			End If
		Wend
	
		stream.Close()
		Return -1		
	End Method
	
	Method UpdateLogin%(username$,userkey$)
		Local url$ = UpdateServer

		url$ = url$.Replace("http://", "")
		Local slashPos% = url$.Find("/"), host$, file$
	
		If slashPos <> -1
			host$ = url$[..slashPos]
			file$ = url$[slashPos..]
		Else
			Return -1
		EndIf
	
		Local stream:TStream = OpenStream("tcp::" + host$)
		If Not stream Then Return -1
	
		Local combined$ = "com=update_login&name="+username+"&key="+userkey
		
		WriteLine(stream,"POST "+file$+" HTTP/1.0")
		WriteLine(stream,"Accept: */*")
		WriteLine(stream,"Host: "+host$)
		WriteLine(stream,"Content-type: application/x-www-form-urlencoded")
		WriteLine(stream,"User-agent: jaqua") 
		WriteLine(stream,"Content-length: " + combined.length)
		WriteLine(stream,"Pragma: no-cache")
		WriteLine(stream,"Connection: close")
		WriteLine(stream,"")
		WriteLine(stream,combined)
		WriteLine(stream,"")

	
		Local check_need% = 0
		
		While Not Eof(stream)
			Local in$ = stream.ReadLine()

			If check_need = 1 Then
				UpdateKey = in
				check_need = 0
			End If
			
			If in$.Find("jControl") <> -1 Then
				check_need = 1
				Continue
			End If
			
			If in$.Find("Fail") <> -1
				RuntimeError "Update Fail (login)"
			End If
		Wend
	
		stream.Close()
		Return 1
	End Method
	Method UpdateUnLogin%()
		Local url$ = UpdateServer

		url$ = url$.Replace("http://", "")
		Local slashPos% = url$.Find("/"), host$, file$
	
		If slashPos <> -1
			host$ = url$[..slashPos]
			file$ = url$[slashPos..]
		Else
			Return -1
		EndIf
	
		Local stream:TStream = OpenStream("tcp::" + host$)
		If Not stream Then Return -1
	
		Local combined$ = "com=update_unlog&key="+UpdateKey
		
		WriteLine(stream,"POST "+file$+" HTTP/1.0")
		WriteLine(stream,"Accept: */*")
		WriteLine(stream,"Host: "+host$)
		WriteLine(stream,"Content-type: application/x-www-form-urlencoded")
		WriteLine(stream,"User-agent: jaqua") 
		WriteLine(stream,"Content-length: " + combined.length)
		WriteLine(stream,"Pragma: no-cache")
		WriteLine(stream,"Connection: close")
		WriteLine(stream,"")
		WriteLine(stream,combined)
		WriteLine(stream,"")

	
		
		While Not Eof(stream)
			Local in$ = stream.ReadLine()

			If in$.Find("Fail") <> -1
				RuntimeError "Update Fail (unlog)"
			End If
		Wend
	
		stream.Close()
		Return 1	
	End Method

	Method Update%()
		Local ok3% = UpgradeAllRes()
		If ok3 = -1 Then Return -1
		
		'Local ok4% = UpdateClientDB()
		'If ok4 = -1 Then Return -1
		
		Return 1
	End Method
	
	Method CheckUpdateAviable%()
		Local ok1% = GetServerRes()
		If ok1 = -1 Then
			'DebugLog "1"
			'RuntimeError "server file"
			Return -1
		End If
		
		Local ok2% = GetClientRes()
		If ok2 = -1 Then Return -1
		
		For Local r:TSystemResource = EachIn Resources
			If r.CanUpdate() = 1 Then
				Info.CanUpdate = 1
				Info.UpdateSize :+ r.GetSize()
				If r.ResourceType = "f" And r.Server_Version > -1 Then Info.TotalFiles:+1
			End If
		Next
		
		If Info.CanUpdate = 1 Then
			Return 1
		Else
			Return 0
		End If
	End Method
	
	Method Free()
		For Local r:TSystemResource = EachIn Resources
			r.Free()
		Next
		Resources.Clear()
		Resources = Null
		
		Info.Free()
		Info = Null
	End Method
	
	Function Create:TSystemUpdate()
		Local upd:TSystemUpdate = New TSystemUpdate
		upd.Info = TSystemUpdateInfo.Create()
		upd.Resources = New TList
		Return upd
	End Function
	
End Type
